angular.module('beamng.apps')
.directive('avgFuelConsumption', ['UiUnits', function (UiUnits) {
  return {
    templateUrl: 'modules/apps/avgFuelConsumption/app.html',
    replace: true,
    restrict: 'EA',
    scope: true,
    controller: ['logger', '$scope', 'bngApi', 'StreamsManager', function (logger, $scope, bngApi, StreamsManager) {
      // Define the list of streams...
      var streamsList = ['electrics', 'engineInfo'];
      // ...and add them to the Stream-Manager.
      StreamsManager.add(streamsList);
      // Make sure to remove the streams when this app is closed.
      $scope.$on('$destroy', function () {
        StreamsManager.remove(streamsList);
      });

      // Set first column descriptions.
      $scope.title1 = 'Traveled distance:';
      $scope.title2 = 'Fuel used:';
      $scope.title3 = 'Fuel Consumption:';
      $scope.title4 = 'Range:';

      // Initialize our data (with remembering the current fuel level).
      var distanceTravelled = 0,
          fuelUsed = 0,
          avgFuelConsumption = 0,
          range = 0,
          fuelLevelLastReset = 0, // We set it to zero, but below I'll take this into account!
          autoResetOk = false,
          timer = 0,
          prevTime = performance.now(),
          curTime = prevTime;
      var TestWheelSpeed;
      
      // Reset function.
      $scope.reset = function ($event) {
        logger.debug('<avg-fuel-consumption> resetting avg. fuel consumption');

        // Reset our data and remember our current fuel level!
        distanceTravelled = 0;
        fuelUsed = 0;
        avgFuelConsumption = 0;
        range = 0;
        fuelLevelLastReset = 0; // We set it to zero, but below I'll take this into account!
        autoResetOk = false;

        timer = 0;
        prevTime = performance.now();
        curTime = prevTime;
      };

      $scope.$on('streamsUpdate', function (event, streams) {
        $scope.$evalAsync(function () {
          if (true
              && (streams.engineInfo != null)
              && (streams.electrics  != null)
             )
          {
            // Retrieve the wheel speed and the current fuel level.
            var wheelSpeed = streams.electrics.wheelspeed;
            var currentFuel = streams.engineInfo[11];

            // Calculate the travelled distance.
            prevTime = curTime;
            curTime = performance.now();
            timer -= 0.001 * (curTime - prevTime);
            if (timer < 0)
            {
              TestWheelSpeed = wheelSpeed;
              // Filtering out the background noise.
              if (wheelSpeed > 0.001)
              { distanceTravelled += ((1.0 - timer) * wheelSpeed); }
              timer = 1;
            }

            // Remembering the fuel level in the reset function didn't work.
            // So there we set it to zero and here we set it to the current level.
            if (true
                && (fuelLevelLastReset == 0)
                && (currentFuel         > 0)
               )
            {
              logger.debug('<avg-fuel-consumption> resetting: reset level is zero!');
              distanceTravelled = 0;
              fuelLevelLastReset = currentFuel;
            }
            // If, out of pure magic, the fuel level suddenly is higher than my remembered level,
            // then HE must have resetted the whole car ==> let's just reset some values then!
            if (true
                && autoResetOk
                && (currentFuel >= fuelLevelLastReset)
               )
            {
              logger.debug('<avg-fuel-consumption> resetting: Fuel level higher than reset level!');
              distanceTravelled = 0;
              fuelLevelLastReset = currentFuel;
              autoResetOk = false; // Allowing this only once, in case the engine was switched off!
            }
            // Allow above automatic resetting, when the fuel level goes below the remembered level.
            // Or when the travelled distance goes above zero!
            if (true
                && !autoResetOk
                && (false
                    || (currentFuel < fuelLevelLastReset)
                    || (distanceTravelled > 0)
                   )
               )
            { autoResetOk = true; }
            // Calculate the currently used fuel.
            fuelUsed = fuelLevelLastReset - currentFuel;

            // Calculate the average fuel consumption rate according to used fuel vs. travelled distance.
            if (distanceTravelled > 0)
            {
              avgFuelConsumption = fuelUsed / distanceTravelled; // l/(s*(m/s)) = l/m
            }
            else
            { avgFuelConsumption = 0; }

            // Calculate the range according to the above fuel consumption rate.
            if (avgFuelConsumption > 0)
            { range = UiUnits.buildString('distance', currentFuel / avgFuelConsumption, 2); }
            else
            {
              if (streams.electrics.wheelspeed > 0.1)
              { range = 'Infinity'; }
              else
              { range = UiUnits.buildString('distance', 0); }
            }

            // Output of calculated data.
            $scope.data1 = UiUnits.buildString('distance', distanceTravelled, 1);
            $scope.data2 = UiUnits.buildString('volume', fuelUsed, 2);
            $scope.data3 = UiUnits.buildString('consumptionRate', avgFuelConsumption, 1);
            $scope.data4 = range;

            // Debug-Text.
            // $scope.debugtext1 = TestWheelSpeed;
            // $scope.debugtext2 = currentFuel;
          }
          else
          { return; }
        });
      });
    }]
  };
}]);
